clear all;

//Read Signal
[s,s_info] = loadwave('speech.wav'); //Read sound-signal
Fs = s_info(3);  //Sampling rate
Bt = s_info(6);  //Quantization bit number
M = s_info(8);  //The number of samples of sound
[d,d_info]=loadwave('noise.wav');  //Read noise-signal
Z = d_info(8);  //The number of samples of noise

if Z<M then
  s=wavread('speech.wav',Z);
  M=Z;
else
  d=wavread('noise.wav',M);
end

x=s+d;
wavwrite(x,Fs,Bt,'input.wav'); //Save x(input-signal) as input.wav

//Initial setup
N = 256;  //FFTsize
h = window('hn',N);  //Set a humming window
N2 = N/2;  //FFT shift size (half of FFTsize)
L = floor(M/N2)-1;  //The number of times of FFT execution   to all samples
C = 1.0;  //Over-subtraction coefficient
P = 4;  //The number of the initial frames for noise estimates
B = 0.98;  //Wiener-filter derivation coefficient
mu = 1.74;  //Variable-Speech-PDF distribution coefficient
//nu = 0.126;  //Variable-Speech-PDF distribution coefficient
alpha = 0.05;  //Variable-Speech-PDF distribution coefficient

//Noise estimate loop
Npow = zeros(1,N);  //Initialization of the noise estimate value
for k=1:P
  n = N2*(k-1);  //Update of the start spot of the FFT
  x1 = h.*x(n+1:n+N);  //Multiplication with humming window
  X1 = fft(x1,-1);  //FFT
  Npow = Npow+abs(X1)/P;  //Estimated noise spectrum (Average of the P-frame)
end

y=x(1:P*N2);  //Output x to P frame

//Noise estimate loop <<Noise-cancelling with Variable-Speech-PDF>>
past_tail=zeros(1,N2);  //Initialization of the half overlap addition signal
gm1=zeros(1,N);  //Initialization of the gamma memory sequence
G=zeros(1,N);  //Initialization of the gain sequence
X1_squaring=zeros(1,N); //Initialization of X1_squaring
D1_squaring=zeros(1,N); //Initialization of D1_squaring

D1_squaring = abs(Npow).^2;  //Calculation of D1_squaring
D1_sum = sum(D1_squaring);  //Calculation of D1_sum
for k=P+1:L
  n = N2*(k-1);  //Update of the start spot of the FFT
  x1 = h.*x(n+1:n+N);  //Multiplication with humming window
  X1 = fft(x1,-1);  //FFT
  X1_squaring = abs(X1).^2;  //Calculation of X1_squaring
  X1_sum = sum(X1_squaring);  //Calculation of X1_sum
  nu_tilde = alpha*10*log10(X1_sum/D1_sum);  //Calculation of nu_tilde
  if(nu_tilde>2) then
      nu = 2;
  elseif(nu_tilde<0) then
      nu = 0;
  else
      nu = nu_tilde;
  end
  
  gm = ( abs(X1).^2 )./Npow;  //Derivation of the gamma (ƒÁ) matrix
  for j=1:N  //Derivation of the (ƒÌ)matrix
    if gm(j)>=1 then
      gz(j) = B*gm1(j)*G(j) + (1-B)*( gm(j)-1 );
    else
      gz(j) = B*gm1(j)*G(j);
    end
    if( gm(j)<>0 & gz(j)<>0 ) then
      u(j) = 0.5 - (mu/4) * sqrt( 1 / ( gm(j) * gz(j) ) );  //Calculation of u to use for a gain
    else
      u(j) = 0.5;
    end
  end
  
  //***************************
  //**  Spectrum gain calculation  **
  //***************************
  for j=1:N
    G(j) = u(j) + sqrt( u(j)^2 + (nu/(2*gm(j))) );
  end
  //************************
  
  G = ( G + abs(G) )/2;  //The gain less than 0 assumes it 0
  Y1 = G.*X1;  //Estimated sound spectrum
  y1 = fft(Y1,1);  //IFFT
  y = [y, past_tail + y1(1:N2)];  //y= (The first half of this IFFT) + (The latter half of last IFFT)
  past_tail=y1(N2+1:N);  //Memorize The latter half of this IFFT
  gm1=gm;  //Memorize gamma
end
savewave('output_lotter.wav',y,Fs);
